
package eu.dnetlib.dhp.eosc.model;

import java.io.Serializable;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.github.imifou.jsonschema.module.addon.annotation.JsonSchema;

/**
 * @author miriam.baglioni
 * @Date 02/02/23
 */
/**
 * It extends eu.dnetlib.dhp.dump.oaf.Instance with values related to the community dump. In the Result dump this
 * information is not present because it is dumped as a set of relations between the result and the datasource. -
 * hostedby of type eu.dnetlib.dhp.schema.dump.oaf.KeyValue to store the information about the source from which the
 * instance can be viewed or downloaded. It is mapped against the hostedby parameter of the instance to be dumped and -
 * key corresponds to hostedby.key - value corresponds to hostedby.value - collectedfrom of type
 * eu.dnetlib.dhp.schema.dump.oaf.KeyValue to store the information about the source from which the instance has been
 * collected. It is mapped against the collectedfrom parameter of the instance to be dumped and - key corresponds to
 * collectedfrom.key - value corresponds to collectedfrom.value
 */
public class Instance implements Serializable {

	@JsonSchema(description = "Information about the source from which the instance can be viewed or downloaded.")
	private CfHbKeyValue hostedby;

	@JsonSchema(description = "Information about the source from which the record has been collected")
	@JsonInclude(JsonInclude.Include.NON_NULL)
	private CfHbKeyValue collectedfrom;

	@JsonSchema(description = "Measures computed for this instance, for example Bip!Finder ones")
	private List<Measure> measures;

	private List<ResultPid> pid;

	@JsonSchema(
		description = "All the identifiers other than pids forged by an authorithy for the pid type (i.e. Crossref for DOIs")
	private List<AlternateIdentifier> alternateIdentifier;

	private String license;

	@JsonSchema(description = "The accessRights for this materialization of the result")
	private AccessRight accessright;

	@JsonSchema(
		description = "The specific sub-type of this instance (see https://api.openaire.eu/vocabularies/dnet:result_typologies following the links)")
	private String type;

	@JsonSchema(
		description = "URLs to the instance. They may link to the actual full-text or to the landing page at the hosting source. ")
	private List<String> url;

	@JsonSchema(
		description = "The money spent to make this book or article available in Open Access. Source for this information is the OpenAPC initiative.")
	private APC articleprocessingcharge;

	@JsonSchema(description = "Date of the research product")
	private String publicationdate;// dateofacceptance;

	@JsonSchema(
		description = "If this instance has been peer-reviewed or not. Allowed values are peerReviewed, " +
			"nonPeerReviewed, UNKNOWN (as defined in https://api.openaire.eu/vocabularies/dnet:review_levels)")
	private String refereed; // peer-review status

	private String fulltext;

	private List<String> eoscDsId;

	public List<String> getEoscDsId() {
		return eoscDsId;
	}

	public void setEoscDsId(List<String> eoscId) {
		this.eoscDsId = eoscId;
	}

	public String getFulltext() {
		return fulltext;
	}

	public void setFulltext(String fulltext) {
		this.fulltext = fulltext;
	}

	public String getLicense() {
		return license;
	}

	public void setLicense(String license) {
		this.license = license;
	}

	public AccessRight getAccessright() {
		return accessright;
	}

	public void setAccessright(AccessRight accessright) {
		this.accessright = accessright;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public List<String> getUrl() {
		return url;
	}

	public void setUrl(List<String> url) {
		this.url = url;
	}

	public String getPublicationdate() {
		return publicationdate;
	}

	public void setPublicationdate(String publicationdate) {
		this.publicationdate = publicationdate;
	}

	public String getRefereed() {
		return refereed;
	}

	public void setRefereed(String refereed) {
		this.refereed = refereed;
	}

	public APC getArticleprocessingcharge() {
		return articleprocessingcharge;
	}

	public void setArticleprocessingcharge(APC articleprocessingcharge) {
		this.articleprocessingcharge = articleprocessingcharge;
	}

	public List<ResultPid> getPid() {
		return pid;
	}

	public void setPid(List<ResultPid> pid) {
		this.pid = pid;
	}

	public List<AlternateIdentifier> getAlternateIdentifier() {
		return alternateIdentifier;
	}

	public void setAlternateIdentifier(List<AlternateIdentifier> alternateIdentifier) {
		this.alternateIdentifier = alternateIdentifier;
	}

	public List<Measure> getMeasures() {
		return measures;
	}

	public void setMeasures(List<Measure> measures) {
		this.measures = measures;
	}

	public CfHbKeyValue getHostedby() {
		return hostedby;
	}

	public void setHostedby(CfHbKeyValue hostedby) {
		this.hostedby = hostedby;
	}

	public CfHbKeyValue getCollectedfrom() {
		return collectedfrom;
	}

	public void setCollectedfrom(CfHbKeyValue collectedfrom) {
		this.collectedfrom = collectedfrom;
	}
}
