
package eu.dnetlib.dhp.oa.graph.dump;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.Optional;

import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.FilterFunction;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;

import eu.dnetlib.dhp.eosc.model.Result;
import eu.dnetlib.dhp.oa.graph.dump.eosc.CommunityMap;
import eu.dnetlib.dhp.oa.graph.dump.eosc.Utils;
import eu.dnetlib.dhp.oa.graph.dump.exceptions.CardinalityTooHighException;
import eu.dnetlib.dhp.oa.graph.dump.exceptions.NoAvailableEntityTypeException;
import eu.dnetlib.dhp.schema.oaf.DataInfo;
import eu.dnetlib.dhp.schema.oaf.OafEntity;

/**
 * It fires the execution of the actual dump for result entities. If the dump is for RC/RI products its checks for each
 * result its belongingess to at least one RC/RI before "asking" for its mapping.
 */
public class DumpProducts implements Serializable {

	public void run(Boolean isSparkSessionManaged, String inputPath, String outputPath, String communityMapPath,
		Class<? extends OafEntity> inputClazz) {

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, outputPath);
				execDump(
					spark, inputPath, outputPath, communityMapPath, inputClazz);
			});
	}

	public static <I extends OafEntity> void execDump(
		SparkSession spark,
		String inputPath,
		String outputPath,
		String communityMapPath,
		Class<I> inputClazz) {

		CommunityMap communityMap = Utils.getCommunityMap(spark, communityMapPath);

		Utils
			.readPath(spark, inputPath, inputClazz)
			.map((MapFunction<I, Result>) value -> execMap(value, communityMap), Encoders.bean(Result.class))
			.filter((FilterFunction<Result>) value -> value != null)
			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(outputPath);

	}

	private static <I extends OafEntity> Result execMap(I value,
		CommunityMap communityMap) throws NoAvailableEntityTypeException, CardinalityTooHighException {

		Optional<DataInfo> odInfo = Optional.ofNullable(value.getDataInfo());
		if (odInfo.isPresent()) {
			if (odInfo.get().getDeletedbyinference() || odInfo.get().getInvisible()) {
				return null;
			}
		} else {
			return null;
		}

		return ResultMapper.map(value, communityMap, null);

	}
}
