
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.Optional;

import org.apache.commons.io.IOUtils;
import org.apache.hadoop.io.Text;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.eosc.model.Indicator;
import eu.dnetlib.dhp.eosc.model.Result;
import eu.dnetlib.dhp.eosc.model.UsageCounts;
import eu.dnetlib.dhp.schema.action.AtomicAction;
import scala.Tuple2;

/**
 * @author miriam.baglioni
 * @Date 07/11/22
 */
public class ExtendWithUsageCounts implements Serializable {

	private static final Logger log = LoggerFactory.getLogger(ExtendEoscResultWithOrganizationStep2.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				ExtendEoscResultWithOrganizationStep2.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/eosc_extend_result_with_indicators_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String actionSetPath = parser.get("actionSetPath");
		log.info("actionSetPath: {}", actionSetPath);

		final String resultPath = parser.get("resultPath");
		log.info("resultPath: {}", resultPath);

		final String outputPath = parser.get("outputPath");
		log.info("outputPath: {}", outputPath);

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, outputPath);
				addIndicators(spark, actionSetPath, outputPath, resultPath);
			});
	}

	private static void addIndicators(SparkSession spark, String actionSetPath, String outputPath, String resultPath) {
		final JavaSparkContext sc = new JavaSparkContext(spark.sparkContext());
		JavaRDD<eu.dnetlib.dhp.schema.oaf.Result> resultsWithIndicatorsRdd = sc
			.sequenceFile(actionSetPath, Text.class, Text.class)
			.map(value -> new ObjectMapper().readValue(value._2().toString(), AtomicAction.class))
			.map(aa -> ((eu.dnetlib.dhp.schema.oaf.Result) aa.getPayload()));

		Dataset<eu.dnetlib.dhp.schema.oaf.Result> resultWithIndicators = spark
			.createDataset(resultsWithIndicatorsRdd.rdd(), Encoders.bean(eu.dnetlib.dhp.schema.oaf.Result.class));

		Dataset<Result> result = Utils
			.readPath(spark, resultPath, Result.class);

		result
			.joinWith(resultWithIndicators, result.col("id").equalTo(resultWithIndicators.col("id")), "left")
			.map((MapFunction<Tuple2<Result, eu.dnetlib.dhp.schema.oaf.Result>, Result>) t2 -> {
				if (Optional.ofNullable(t2._2()).isPresent()) {
					Indicator indicator = new Indicator();
					UsageCounts uc = new UsageCounts();
					t2._2().getMeasures().stream().forEach(m -> {
						if (m.getId().equals("downloads")) {
							uc.setDownloads(m.getUnit().get(0).getValue());
						} else {
							uc.setViews(m.getUnit().get(0).getValue());
						}
						indicator.setUsageCounts(uc);
						t2._1().setIndicator(indicator);
					});

				}
				return t2._1();
			}, Encoders.bean(Result.class))
			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(outputPath);

	}
}
