
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Optional;

import javax.rmi.CORBA.Util;

import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.api.java.function.MapGroupsFunction;
import org.apache.spark.sql.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.eosc.model.Relation;
import eu.dnetlib.dhp.eosc.model.Result;
import scala.Tuple2;

/**
 * @author miriam.baglioni
 * @Date 02/02/23
 */
public class SparkExtendResultWithRelation implements Serializable {

	private static final Logger log = LoggerFactory.getLogger(SparkExtendResultWithRelation.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				SparkExtendResultWithRelation.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/eosc_input_extendwithrelation_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String resultPath = parser.get("resultPath");
		log.info("resultPath: {}", resultPath);

		final String relationPath = parser.get("relationPath");
		log.info("relationPath: {}", relationPath);

		final String outputPath = parser.get("outputPath");
		log.info("outputPath: {}", outputPath);

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, outputPath);
				extendResultWithRelation(spark, resultPath, relationPath, outputPath);

			});

	}

	private static void extendResultWithRelation(SparkSession spark, String resultPath, String relationPath,
		String outputPath) {
		Dataset<Relation> relationDataset = Utils.readPath(spark, relationPath, Relation.class);
		Dataset<Result> resultDataset = Utils.readPath(spark, resultPath, Result.class);

		resultDataset
			.joinWith(relationDataset, resultDataset.col("id").equalTo(relationDataset.col("source")), "left")
			.groupByKey((MapFunction<Tuple2<Result, Relation>, String>) t2 -> t2._1().getId(), Encoders.STRING())
			.mapGroups((MapGroupsFunction<String, Tuple2<Result, Relation>, Result>) (k, it) -> {
				Tuple2<Result, Relation> first = it.next();
				Result r = first._1();
				if (Optional.ofNullable(first._2()).isPresent()) {
					if (r.getRelations() == null) {
						r.setRelations(new ArrayList<>());
					}
					r.getRelations().add(first._2());
					it.forEachRemaining(t2 -> r.getRelations().add(t2._2()));
				}
				return r;
			}, Encoders.bean(Result.class))
			.write()
			.option("compression", "gzip")
			.mode(SaveMode.Overwrite)
			.json(outputPath);
	}
}
