
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.*;

import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.FilterFunction;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.eosc.model.Provenance;
import eu.dnetlib.dhp.eosc.model.RelType;
import eu.dnetlib.dhp.eosc.model.Result;
import eu.dnetlib.dhp.schema.oaf.*;
import scala.Tuple2;

/**
 * @author miriam.baglioni
 * @Date 12/01/23
 */
public class SparkSelectRelation implements Serializable {
	private static final Logger log = LoggerFactory.getLogger(SparkSelectRelation.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				SparkSelectRelation.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/eosc_input_relationdump_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String inputPath = parser.get("sourcePath");
		log.info("inputPath: {}", inputPath);

		final String outputPath = parser.get("outputPath");
		log.info("outputPath: {}", outputPath);

		Optional<String> rs = Optional.ofNullable(parser.get("removeSet"));
		final Set<String> removeSet = new HashSet<>();
		if (rs.isPresent()) {
			Collections.addAll(removeSet, rs.get().split(";"));
		}

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				selectSubset(spark, inputPath, outputPath, removeSet);

			});

	}

	private static void selectSubset(SparkSession spark, String inputPath, String outputPath, Set<String> removeSet) {
		Dataset<Relation> relation = Utils
			.readPath(spark, inputPath + "/relation", Relation.class)
			.filter(
				(FilterFunction<Relation>) r -> !r.getDataInfo().getDeletedbyinference()
					&& !removeSet.contains(r.getRelClass()));

		Dataset<Tuple2<String, String>> resultIds = Utils
			.readPath(spark, outputPath + "/publication", Result.class)
			.map(
				(MapFunction<Result, Tuple2<String, String>>) p -> new Tuple2<>(p.getId(), p.getType()),
				Encoders.tuple(Encoders.STRING(), Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, outputPath + "/dataset", Result.class)
					.map(
						(MapFunction<Result, Tuple2<String, String>>) d -> new Tuple2<>(d.getId(), d.getType()),
						Encoders.tuple(Encoders.STRING(), Encoders.STRING())))
			.union(
				Utils
					.readPath(spark, outputPath + "/software", Result.class)

					.map(
						(MapFunction<Result, Tuple2<String, String>>) s -> new Tuple2<>(s.getId(), s.getType()),
						Encoders.tuple(Encoders.STRING(), Encoders.STRING())))
			.union(
				Utils
					.readPath(spark, outputPath + "/otherresearchproduct", Result.class)

					.map(
						(MapFunction<Result, Tuple2<String, String>>) o -> new Tuple2<>(o.getId(), o.getType()),
						Encoders.tuple(Encoders.STRING(), Encoders.STRING())));

		// select result -> result relations
		Dataset<Relation> relResultResult = relation
			.joinWith(resultIds, relation.col("source").equalTo(resultIds.col("_1")))
			.map(
				(MapFunction<Tuple2<Relation, Tuple2<String, String>>, Relation>) t2 -> t2._1(),
				Encoders.bean(Relation.class));

		relResultResult
			.joinWith(resultIds, relResultResult.col("target").equalTo(resultIds.col("_1")))
			.map((MapFunction<Tuple2<Relation, Tuple2<String, String>>, eu.dnetlib.dhp.eosc.model.Relation>) t2 -> {
				eu.dnetlib.dhp.eosc.model.Relation relNew = new eu.dnetlib.dhp.eosc.model.Relation();
				Relation rel = t2._1();
				relNew
					.setSource(

						rel.getSource());

				relNew
					.setTarget(

						rel.getTarget());

				relNew
					.setReltype(
						RelType
							.newInstance(
								rel.getRelClass(),
								rel.getSubRelType()));
				relNew.setTargetType(t2._2()._2());

				Optional<DataInfo> odInfo = Optional.ofNullable(rel.getDataInfo());
				if (odInfo.isPresent()) {
					DataInfo dInfo = odInfo.get();
					if (Optional.ofNullable(dInfo.getProvenanceaction()).isPresent() &&
						Optional.ofNullable(dInfo.getProvenanceaction().getClassname()).isPresent()) {
						relNew
							.setProvenance(
								Provenance
									.newInstance(
										dInfo.getProvenanceaction().getClassname(),
										dInfo.getTrust()));
					}
				}

				return relNew;

			}, Encoders.bean(eu.dnetlib.dhp.eosc.model.Relation.class))
			.write()
			.option("compression", "gzip")
			.mode(SaveMode.Overwrite)
			.json(outputPath + "/relation");

	}

}
